#include "sys/CMS_machine.h"
#include "pilot.h"
*CMZ :          18/02/96  17.35.51  by  Lucas Taylor
*-- Author :    Lucas Taylor   14/07/95
*+DOC*******************************************************************
       PROGRAM KI_EDDE_MAIN
*
*DOC   Action:    Main steering routine for EDDE in CMKIN package
*
*DOC   Called by: System
*
*DOC   Input:     None
*DOC   Output:    None
*
*DOC   Author:    Lucas Taylor, Veikko Karimaki
*DOC   Created:   14/07/95
*DOC   Author:    Andrei Sobol
*DOC   Created:   09/02/05
*
*+DOC*******************************************************************
#include "cmsi/cmcdes/impnone.inc"
#include "cmsi/cmcdes/chepevt.inc"
#include "cmsi/cmcdes/mc_param.inc"
#include "kine/kicdes/kikeys.inc"
#include "kine/kicdes/kicwn.inc"
*
      INTEGER I
*
      INTEGER ISEL,IFILE
      SAVE    ISEL,IFILE
*
      LOGICAL I_WANT, DUMMY
*-----------------------------------------------------------------------
*
*     Initialisation phase
*     --------------------
*
      CALL KI_INIT (0)              ! CMKIN initialisation (data cards, etc.)
*
      CALL KI_EVT_SEL (-1,DUMMY)    ! KSEL-dependent selection initialisation
*
      CALL EDDEINI                  ! EDDE initialisation
*** tauola usage
      IF (JAKK1.GE.0)
     &CALL PRETAUOLA(-1)    
*
      ISEL = 0                      ! Initialise selected event counter
      IFILE = 0
      IF (N_NFIL.EQ.-99) N_NFIL=N_NSEL
*
*-----------------------------------------------------------------------
*
*     Event generation
*     ----------------
*
      DO I=1,N_TRIG                 ! Event generation loop
        NHEP = 0   
        NVRMCP = 0
        CALL EDDEEVE                ! Generate EDDE event
*
	IF(I.LE.1) CALL PYLIST(1)
c        CALL PYLIST(1)
*
** tauola usage
        IF (JAKK1.GE.0)
     &  CALL PRETAUOLA(0)    

cc        CALL KI_EVT_SEL (0,I_WANT)  ! KSEL-dependent event selection
ccc       CALL KIS_EDDE (0,I_WANT)  ! KSEL-dependent event selection
*
        IF (I_WANT) THEN            ! Event was selected
          ISEL = ISEL + 1
	  IF (NHEP.EQ.0)
     &    CALL KIU_FILL_HEPEVT      ! Fill HEPEVT common block
          IF (IDCWN.GT.0) THEN
            CALL KI_WR_EVT          ! Save event to ntuple file
          ELSE
            CALL KIR_ANAL
          ENDIF
*
	  IF (MOD(ISEL,100).EQ.1.OR.ISEL.EQ.N_NSEL.OR.MOD(ISEL,N_NFIL).EQ.0)
     &    WRITE(*,*)
     &    '**KIMAIN** Nb. events triggered, selected =',I,ISEL
*
          IF (ISEL.EQ.N_NSEL) GO TO 99   ! End of job	
          IF (MOD(ISEL,N_NFIL).EQ.0.AND.IDCWN.GT.0) THEN
            CALL CLOSE_NTUPLE
            IFILE = IFILE + 1
            CALL RENAME_NTUPLE(IFILE)
            CALL OPEN_NEW_NTUPLE
          ENDIF
        ENDIF	
      ENDDO
*
*-----------------------------------------------------------------------
*
*     Termination phase
*     -----------------
*
   99 CONTINUE
*
      CALL KI_EVT_SEL (1,DUMMY)     ! KSEL-dependent selection termination
*
      CALL PYSTAT(1)                ! Write-out PYTHIA run summary
*
      IF (JAKK1.GE.0) CALL PRETAUOLA(1)  ! srs insertion
      IF (IDCWN.GT.0) THEN
        CALL CLOSE_NTUPLE
        IF (ISEL.GT.N_NFIL) THEN
          IFILE = IFILE + 1
          CALL RENAME_NTUPLE(IFILE)
        ENDIF
      ELSE
        CALL KIR_END
      ENDIF
*
      CALL KI_END(0)                ! CMKIN termination
*-----------------------------------------------------------------------
      END                           ! End of KI_EDDE_MAIN

C
C-+++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c----------------------------------------------------------------------              
c-*... EDDE initialization ! A.Sobol,R.Ryutin
c----------------------------------------------------------------------              
      SUBROUTINE EDDEINI
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)
      IMPLICIT INTEGER(I-N)
#include "kine/kicdes/kikeys.inc"
C-... model parameters
      EXTERNAL EDDEDATA
c-...global EDDE parameters
c      INTEGER     MXGLPAR
c      INTEGER IPARS
c      REAL RPARS
c      PARAMETER   (MXGLPAR=200)
c      COMMON /KI_TXPAR/ IPARS(MXGLPAR), RPARS(MXGLPAR)
c-...some EDDE variables
c------ fundamental constants -----------------------------
      INTEGER NF,NC,NLOSW
      DOUBLE PRECISION PI,CSMB,LAMQCD,
     & TF,CF,BF0,BF1
      DOUBLE COMPLEX MNI,REI
      COMMON/EDDEFUND/ MNI,REI,PI,CSMB,LAMQCD,
     & TF,CF,BF0,BF1,NF,NC,NLOSW    
c------ parameters for soft rescattering (trajectories)----
c------ (t1,t2,fi0 dependence) ----------------------------
      INTEGER NAPR,NFI
      DOUBLE PRECISION CP,DP,RP,RG,AP,
     &   T1MIN,T1MAX,T2MIN,T2MAX,FKK,CGP
      COMMON/EDDESOFT/ CP(3),DP(3),RP(3),RG(3),AP(3),
     &   T1MIN,T1MAX,T2MIN,T2MAX,FKK,CGP,NAPR,NFI 
c----- parameters to calculate overall s-dependence -------
      DOUBLE PRECISION XI1MIN,XI2MIN,XI1MAX,XI2MAX
      COMMON/EDDETOT/ XI1MIN,XI2MIN,XI1MAX,XI2MAX
c----- parameters for hard cross-sections -----------------
      DOUBLE PRECISION MGGCUT,ETJCUT,MXMAX,
     & ETAJMAX,PLUM,PSURV,PSUD,ETASIMAX,SQS,
     & PSIDD1,PSIDD2
      COMMON/EDDEHARD/ MGGCUT,ETJCUT,MXMAX,
     & ETAJMAX,PLUM,PSURV,PSUD,ETASIMAX,SQS,
     & PSIDD1,PSIDD2
c--- restrictions on the phase space of g-jet ---
c--- DER3J - max. angle between g-jet and parallel jet ----
c--- XMAX3J - max ratio 2*Eg/MJJ --------------------------
c--- parameters for 3g functions -----------------------------
      DOUBLE PRECISION DER3J,XMAX3J,PAR3G
      COMMON/EDDE3JP/ DER3J,XMAX3J,PAR3G(5)     
c----- parameters to calculate total cross-sections -------      
c----- RS1 parameters -------------------------------------
      INTEGER NRS0 
      DOUBLE PRECISION RSXI0,RSGAM0,RSMH0,RSMR0 
      COMMON/EDDERS1/ RSXI0,RSGAM0,RSMH0,RSMR0,NRS0
c----- additional global parameters -----
      INTEGER KCP,IPROC
      DOUBLE PRECISION AM0,AMP,S,MQ
      COMMON/EDDEOTHER/ KCP,IPROC,AM0,AMP,S,MQ
c-...some EDDE functions
      DOUBLE PRECISION EDDECS
c-*...standard PYTHIA ( v. >= 6.2) commons for initialization
      EXTERNAL PYDATA
      INTEGER  PYCOMP
      COMMON /PYDAT1/ MSTU(200),PARU(200),MSTJ(200),PARJ(200)
      COMMON /PYDAT2/ KCHG(500,4),PMAS(500,4),PARF(2000),VCKM(4,4)
      COMMON /PYDAT3/ MDCY(500,3),MDME(8000,2),BRAT(8000),KFDP(8000,5)
      COMMON /PYSUBS/ MSEL,MSELPD,MSUB(500),KFIN(2,-40:40),CKIN(200)
      COMMON /PYPARS/ MSTP(200),PARP(200),MSTI(200),PARI(200)
      COMMON /PYINT5/ NGENPD,NGEN(0:500,3),XSEC(0:500,3)
      COMMON /PYDATR/ MRPY(6),RRPY(100)  
C-...User process initialization commonblock.
      INTEGER MAXPUP
      PARAMETER (MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON/HEPRUP/IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),XMAXUP(MAXPUP),
     &LPRUP(MAXPUP)
      SAVE /HEPRUP/
C      
c      CALL KI_PYTS2PYTD(1)         ! Interface between Pythia single ...
c      CALL EDDEFFR                 ! Read the data cards
cc      CALL EDDEFFRTEST                 ! Read the data cards
c      CALL KI_PYTS2PYTD(2)         ! ... DOUBLE PRECISION

      CALL EDDEPUTDAT ! to include the data
      CALL EDDETITLE ! title page for the generator

      IPROC =IPARS(1)
      KCP   =IPARS(4)
      NFI   =IPARS(5)
c      SQS   =RPARS(1)
      SQS   =ENERGY_CMS
      print*,'------------------------------>',ENERGY_CMS
      S     =SQS*SQS
      MTBCUT=RPARS(2)
      PKMIN = 4.D0*MTBCUT*MTBCUT/S
      AM0   =RPARS(3)                                !central particle mass
      AMP   =PMAS(PYCOMP(2212),1)                    !proton mass
C
      IDBMUP(1) = 2212 
      IDBMUP(2) = 2212 
      EBMUP(1)  = SQS/2
      EBMUP(2)  = SQS/2
      IDWTUP    = 1
      IF(IPROC.EQ.400)   IDWTUP    = 3
c      IF(IPROC.EQ.406)   IDWTUP    = 3   !soboladd
      NPRUP     = 1
      LPRUP(1)  = IPROC
      XSECUP(1) = 1.D0 
      XMAXUP(1) = 1.D0 
C--
      PARI(1)   = EDDECS(IPROC)*1.D-12      ! cross section in mb
      PARI(7)   = 1.D0 
      IF(IPROC.EQ.400) PMAS(IPARS(4),1)=AM0          ! Higgs mass 
c      IF(IPROC.EQ.406) PMAS(IPARS(4),1)=AM0          ! Higgs mass soboladd
C--
      CALL PYINIT('USER','p','p',SQS)
      XSEC(4,3) = PARI(1)
      NGEN(4,3) = 0
      WRITE(*,*)' CS(',IPROC,')=',PARI(1)*1.D+12,' fb'

      RETURN
      END
*-- Author : A.Sobol
*
*
*
************************************************************************
      SUBROUTINE KIS_EDDE (IFLG,IWNT)
*                                                                      *
************************************************************************
      IMPLICIT NONE
#include "kine/kicdes/kikeys.inc"
#include "kine/kicdes/kikfcodes.inc"
#include "kine/kicdes/pythia.inc"
      DOUBLE PRECISION PYR,PYMASS
      REAL PT1,PT2,PT3
      INTEGER IFLG
      LOGICAL IWNT
*
      IF (IFLG.EQ.-1) THEN

        PRINT *
        PRINT *,'*** INITIALIZING KIS_EDDE'
        PRINT *
	
      ENDIF
*
      IF (IFLG.NE.0) GO TO 999
      IWNT = .TRUE.
*
      IF(IPARS(1).EQ.500) THEN
       IF(N.LE.10) IWNT = .FALSE.
      ENDIF
      IF(IPARS(1).EQ.400) THEN
       PT1=DSQRT(P(6,1)**2+P(6,2)**2)
       PT2=DSQRT(P(7,1)**2+P(7,2)**2)
       IF(PT1.LE.RPARS(4).OR.PT2.LE.RPARS(4)) IWNT = .FALSE.
      ENDIF
      IF(IPARS(1).EQ.401) THEN
       PT1=DSQRT(P(6,1)**2+P(6,2)**2)
       PT2=DSQRT(P(7,1)**2+P(7,2)**2)
       IF(PT1.LE.RPARS(4).OR.PT2.LE.RPARS(4)) IWNT = .FALSE.
      ENDIF
      IF(IPARS(1).EQ.402) THEN
       PT1=DSQRT(P(6,1)**2+P(6,2)**2)
       PT2=DSQRT(P(7,1)**2+P(7,2)**2)
       IF(PT1.LE.RPARS(4).OR.PT2.LE.RPARS(4)) IWNT = .FALSE.
      ENDIF
      IF(IPARS(1).EQ.403) THEN
       PT1=DSQRT(P(6,1)**2+P(6,2)**2)
       PT2=DSQRT(P(7,1)**2+P(7,2)**2)
       PT3=DSQRT(P(8,1)**2+P(8,2)**2)
c       IF(PT1.LE.RPARS(4).OR.PT2.LE.RPARS(4).OR.PT3.LE.RPARS(4))
c     &                                            IWNT = .FALSE.
       IF(PT1.LE.RPARS(4).OR.PT3.LE.RPARS(4))
     &                                            IWNT = .FALSE.
      ENDIF
      
      
  999 CONTINUE
      END
C-====== call the data!!! ====================================
      SUBROUTINE EDDEPUTDAT
      
      IMPLICIT NONE
    
C-... model parameters
      EXTERNAL EDDEDATA,EDDETABLES
#include "kine/kicdes/kikeys.inc"
c-...global EDDE parameters from FFR file
c      INTEGER MXGLPAR
c      INTEGER IPARS
c      REAL RPARS
c      PARAMETER   (MXGLPAR=200)
c      COMMON /KI_TXPAR/ IPARS(MXGLPAR), RPARS(MXGLPAR)
      
      INTEGER I,J,IJ
      DOUBLE PRECISION AUX1,AUX2,AUX3
c---- tables ------------------------------------      
c---- table for Intfgg**2 for Lum_sidde=Lum_excl*Intfgg**2 ---
      DOUBLE PRECISION LUM1,FLUM1,DX1,DY1,X01,Y01
      COMMON/EDDETAB1/ LUM1(480),FLUM1(30,16),
     & DX1,DY1,X01,Y01
c---- table for 3g ratio dIsud3g(x,mu)**2/Isud0(mu)**2 -------
      DOUBLE PRECISION RDI3G,FRDI3G,DX2,DY2,X02,Y02
      COMMON/EDDETAB2/ RDI3G(630),FRDI3G(30,21),
     & DX2,DY2,X02,Y02 
c--- table for 3g ratio Isud3ga(etasimax,mu)**2/Isud0(mu)**2 -
c--- table for 3g ratio Isud3gb(etasimax,mu)**2/Isud0(mu)**2 -
      DOUBLE PRECISION RI3GA,RI3GB,FRI3GA,FRI3GB,
     & DX3,DY3,X03,Y03
      COMMON/EDDETAB3/ RI3GA(480),RI3GB(480),
     & FRI3GA(30,16),FRI3GB(30,16),DX3,DY3,X03,Y03
c- tables for fM(x,pt)
       DOUBLE PRECISION FT4,FMT4,DX4,DY4,X04,Y04
       COMMON/EDDETAB4/ FT4(10201),FMT4(101,101),
     & DX4,DY4,X04,Y04 
c-- tables for dfM_g/g(x,Y)/dY, Y=DLOG(PT**2/LAMQCD**2) ---
       DOUBLE PRECISION DFT5,DFMT5,DX5,DY5,X05,Y05
       COMMON/EDDETAB5/ DFT5(10201),DFMT5(101,101),
     & DX5,DY5,X05,Y05     
c------ fundamental constants -----------------------------
      INTEGER NF,NC,NLOSW
      DOUBLE PRECISION PI,CSMB,LAMQCD,
     & TF,CF,BF0,BF1
      DOUBLE COMPLEX MNI,REI
      COMMON/EDDEFUND/ MNI,REI,PI,CSMB,LAMQCD,
     & TF,CF,BF0,BF1,NF,NC,NLOSW    
c------ parameters for soft rescattering (trajectories)----
c------ (t1,t2,fi0 dependence) ----------------------------
      INTEGER NAPR,NFI
      DOUBLE PRECISION CP,DP,RP,RG,AP,
     &   T1MIN,T1MAX,T2MIN,T2MAX,FKK,CGP
      COMMON/EDDESOFT/ CP(3),DP(3),RP(3),RG(3),AP(3),
     &   T1MIN,T1MAX,T2MIN,T2MAX,FKK,CGP,NAPR,NFI 
c----- parameters to calculate overall s-dependence -------
      DOUBLE PRECISION XI1MIN,XI2MIN,XI1MAX,XI2MAX
      COMMON/EDDETOT/ XI1MIN,XI2MIN,XI1MAX,XI2MAX
c----- parameters for hard cross-sections -----------------
      DOUBLE PRECISION MGGCUT,ETJCUT,MXMAX,
     & ETAJMAX,PLUM,PSURV,PSUD,ETASIMAX,SQS,
     & PSIDD1,PSIDD2
      COMMON/EDDEHARD/ MGGCUT,ETJCUT,MXMAX,
     & ETAJMAX,PLUM,PSURV,PSUD,ETASIMAX,SQS,
     & PSIDD1,PSIDD2
c--- restrictions on the phase space of g-jet ---
c--- DER3J - max. angle between g-jet and parallel jet ----
c--- XMAX3J - max ratio 2*Eg/MJJ --------------------------
c--- parameters for 3g functions -----------------------------
      DOUBLE PRECISION DER3J,XMAX3J,PAR3G
      COMMON/EDDE3JP/ DER3J,XMAX3J,PAR3G(5)     
c----- parameters to calculate total cross-sections -------      
c----- RS1 parameters -------------------------------------
      INTEGER NRS0 
      DOUBLE PRECISION RSXI0,RSGAM0,RSMH0,RSMR0 
      COMMON/EDDERS1/ RSXI0,RSGAM0,RSMH0,RSMR0,NRS0
c----- additional global parameters -----
      INTEGER KCP,IPROC
      DOUBLE PRECISION AM0,AMP,S,MQ
      COMMON/EDDEOTHER/ KCP,IPROC,AM0,AMP,S,MQ
c- commons for cascad generation      
      DOUBLE PRECISION FGGQ0,FGGQS
      COMMON/EDDEFGG/FGGQ0,FGGQS
c----- data that needs calculation
c----- f_g/g lower limit and minimal generated PT_soft_g
       FGGQ0=0.3D0
       FGGQS=1.D0
c--- RS1 data from FFR ----------------
       NRS0=IPARS(6) ! FFRDAT IPARS(6)
       RSXI0=RPARS(5) ! FFRDAT RPARS(5)
       RSGAM0=RPARS(6) ! FFRDAT RPARS(6)
       RSMH0=RPARS(7) ! FFRDAT RPARS(7)
       RSMR0=RPARS(8) ! FFRDAT RPARS(8)
c--- other parameters
c--- code of central particle
       KCP=IPARS(4) ! FFRDAT IPARS(4)
c--- code of the process
       IPROC=IPARS(1) ! FFRDAT IPARS(1)
c--- mass of the central particle
       AM0=RPARS(3) ! FFRDAT RPARS(3)
      IF (NRS0.GT.0) THEN
      CALL EDDERS1C(NRS0,RSXI0,RSGAM0,RSMH0,RSMR0,AUX1,AUX2,AUX3)
      AM0=AUX1
      ENDIF
c--- phi dependence code
       NFI=IPARS(5) ! FFRDAT IPARS(5)
c------ constants -----------------------------------------
       CF=(NC*NC-1)*TF/NC
       BF0=(11.D0*NC-2.D0*NF)/(12.D0*PI)
       BF1=34.D0*NC*NC/3.D0-20.D0*NC*NF*TF/3.D0
       BF1=(BF1-4.D0*CF*TF*NF)/(BF0*16.D0*PI*PI)      
c------ cuts for hard c.-s. variables
c- external input SQS ---
       SQS=RPARS(1)  ! FFRDAT RPARS(1)
       S=SQS*SQS
       MQ=4.8D0 ! FFRDAT RPARS(10)
       MXMAX=500.D0
c------ additional par. for c.-s. calculations ------------
       FKK=0.0196721D0+0.737705D-06*SQS 
c- external input MGGCUT -------
c- cut on the central mass ----- 
       ETJCUT=RPARS(2) ! FFRDAT RPARS(2)
       MGGCUT=2.D0*ETJCUT
       ETAJMAX=DLOG(MXMAX/MGGCUT+DSQRT((MXMAX/MGGCUT)**2-1))
c- external input ETASIMAX -----------------
c- cut on semi-incl. |eta_smth|<ETASIMAX/2 -
c- rap-gap constraint ----------------------
c- parameter is relevant only for exclusive 3j case,
c- SIDDE calculations for all rapidities by def.
       ETASIMAX=RPARS(9) ! FFRDAT RPARS(9)     
       PSURV=0.66D-01
       PSUD=0.6795D0 ! for exclusive case
c--- calculation of estimation pars for ISUDGEN2 -------------
c-       IF (ETASIMAX.LE.0.D0) THEN
c-        PSIDD1=0.D0
c-        PSIDD2=0.D0
c-       ENDIF 
c-       IF (ETASIMAX.LE.2.D0.AND.ETASIMAX.GT.0.D0) THEN
c-        PSIDD1=(0.9D-05+0.5D0*ETASIMAX*(0.02023D0-0.9D-05))*5.D0**0.7D0
c-        PSIDD2=-0.7D0
c-       ENDIF
c-       IF (ETASIMAX.LE.4.D0.AND.ETASIMAX.GT.2.D0) THEN
c-        PSIDD1=(0.02023D0+0.123D0*(ETASIMAX-2.D0))*5.D0**0.7D0
c-        PSIDD2=-0.7D0
c-       ENDIF
c-       IF (ETASIMAX.LE.6.D0.AND.ETASIMAX.GT.4.D0) THEN
c-        PSIDD1=(0.215D0+0.3D0*(ETASIMAX-4.D0))*5.D0**0.5D0
c-        PSIDD2=-0.5D0
c-       ENDIF       
c-       IF (ETASIMAX.LE.8.D0.AND.ETASIMAX.GT.6.D0) THEN
c-        PSIDD1=(0.5D0+0.27D0*(ETASIMAX-6.D0))*5.D0**0.1D0
c-        PSIDD2=-0.1D0
c-       ENDIF
c-       IF (ETASIMAX.LE.12.D0.AND.ETASIMAX.GT.8.D0) THEN
c-        PSIDD1=(0.673D0+0.17D0*(ETASIMAX-8.D0))/5.D0**0.35D0
c-        PSIDD2=0.35D0
c-       ENDIF
c-       IF (ETASIMAX.LE.16.D0.AND.ETASIMAX.GT.12.D0) THEN
c-        PSIDD1=(1.032D0+0.05D0*(ETASIMAX-12.D0))/5.D0**0.6D0
c-        PSIDD2=0.6D0
c-       ENDIF
c-       IF (ETASIMAX.GT.16.D0) THEN
c-        PSIDD1=0.9173/5.D0**0.8D0
c-        PSIDD2=0.8D0
c-       ENDIF
c- all the above PSIDDN for old semi-inclusive estimations.
c- for new sidde etamax->Infinity,is not required
       PSIDD1=0.112732D-01
       PSIDD2=1.5D0
c- overall gg-lum. negative power in MJJ for the dsig/dMJJ --- 
       PLUM=4.D0*DP(3)+1.D0+PSURV+PSUD-PSIDD2
c--- data for 3 jets
c--- model restrictions on the phase space of g-jet -------
c--- DER3J - max. angle between g-jet and parallel jet ----
c--- XMAX3J - max ratio 2*Eg/MJJ --------------------------
       DER3J=0.75D0
       XMAX3J=0.72D0
       PAR3G(1)=0.22D0
       PAR3G(2)=(0.13D0*DLOG(DCOSH(ETASIMAX/2.D0))-0.2D0)
c---
       AUX1=0.61D0+2.42D0*(ETASIMAX*0.5D0-0.5D0)**0.7D0
        IF (ETASIMAX.GE.4.4D0.AND.ETASIMAX.LT.7.D0) THEN
         AUX1=4.16D0 
        ENDIF 
        IF (ETASIMAX.GE.7.D0.AND.ETASIMAX.LT.15.D0) THEN
         AUX1=1.D0+5.65D0/(0.5D0*ETASIMAX-2.D0)**1.6D0
        ENDIF
        IF (ETASIMAX.GE.15.D0) THEN
         AUX1=1.D0+5.65D0/5.5D0**1.6D0
        ENDIF
        IF (ETASIMAX.GT.10.D0) THEN
         AUX1=AUX1-0.2D0
        ENDIF
        PAR3G(3)=AUX1
c---
        AUX2=0.128D0*ETASIMAX/2.D0
        IF (ETASIMAX.GT.4.D0) THEN
         AUX2=AUX2+0.01D0*ETASIMAX/2.D0-0.09D0
        ENDIF
        IF (ETASIMAX.GT.8.D0) THEN
         AUX2=AUX2-0.05D0*ETASIMAX/2.D0-0.08D0
        ENDIF
        IF (ETASIMAX.GT.12.D0) THEN
         AUX2=AUX2-0.1D0*(ETASIMAX/2.D0-6.D0)-0.21D0
        ENDIF
        IF (ETASIMAX.GT.15.D0) THEN
         AUX2=AUX2-0.1D0*(15.D0/2.D0-6.D0)-0.21D0
        ENDIF        
        PAR3G(4)=AUX2
c---
        AUX3=0.2D0
        IF (ETASIMAX.GT.4.D0) THEN
         AUX3=AUX3+0.12D0
        ENDIF
        IF (ETASIMAX.GT.8.D0) THEN
         AUX3=AUX3+0.3D0
        ENDIF
        IF (ETASIMAX.GT.12.D0) THEN
         AUX3=AUX3+0.2D0
        ENDIF
        PAR3G(5)=AUX3

c--- manipulation with tables
       I=1
       J=1
       IJ=1
       DO I=1,30
         DO J=1,16
           FLUM1(I,J)=LUM1(IJ) 
           FRI3GA(I,J)=RI3GA(IJ)
           FRI3GB(I,J)=RI3GB(IJ)
           IJ=IJ+1
         ENDDO
       ENDDO
       
       I=1
       J=1
       IJ=1
       DO I=1,30
         DO J=1,21 
           FRDI3G(I,J)=RDI3G(IJ)
           IJ=IJ+1
         ENDDO
       ENDDO       

       I=1
       J=1
       IJ=1
       DO I=1,101
         DO J=1,101
           FMT4(I,J)=FT4(IJ) 
           DFMT5(I,J)=DFT5(IJ)
           IJ=IJ+1
         ENDDO
       ENDDO

      RETURN
      END
c----------------------------------------------------------------------              
c-*... jet with flat E vs eta distr. /A.Sobol
c----------------------------------------------------------------------              
      SUBROUTINE EDDEFLAT
      IMPLICIT DOUBLE PRECISION(A-H, O-Z)
      IMPLICIT INTEGER(I-N)
#include "kine/kicdes/kikeys.inc"
c-*...standard PYTHIA ( v. >= 6.2) commons for initialization
      INTEGER PYCOMP
      COMMON /PYJETS/ NN, NPAD, KK(4000,5), PP(4000,5), VV(4000,5)
      COMMON /PYDAT2/ KCHG(500,4),PMAS(500,4),PARF(2000),VCKM(4,4)
      COMMON /PYINT5/ NGENPD,NGEN(0:500,3),XSEC(0:500,3)
      COMMON /PYPARS/ MSTP(200),PARP(200),MSTI(200),PARI(200)
c-...some EDDE variables
c------ fundamental constants -----------------------------
      INTEGER NF,NC,NLOSW
      DOUBLE PRECISION PI,CSMB,LAMQCD,
     & TF,CF,BF0,BF1
      DOUBLE COMPLEX MNI,REI
      COMMON/EDDEFUND/ MNI,REI,PI,CSMB,LAMQCD,
     & TF,CF,BF0,BF1,NF,NC,NLOSW    
c------ parameters for soft rescattering (trajectories)----
c------ (t1,t2,fi0 dependence) ----------------------------
      INTEGER NAPR,NFI
      DOUBLE PRECISION CP,DP,RP,RG,AP,
     &   T1MIN,T1MAX,T2MIN,T2MAX,FKK,CGP
      COMMON/EDDESOFT/ CP(3),DP(3),RP(3),RG(3),AP(3),
     &   T1MIN,T1MAX,T2MIN,T2MAX,FKK,CGP,NAPR,NFI 
c----- parameters to calculate overall s-dependence -------
      DOUBLE PRECISION XI1MIN,XI2MIN,XI1MAX,XI2MAX
      COMMON/EDDETOT/ XI1MIN,XI2MIN,XI1MAX,XI2MAX
c----- parameters for hard cross-sections -----------------
      DOUBLE PRECISION MGGCUT,ETJCUT,MXMAX,
     & ETAJMAX,PLUM,PSURV,PSUD,ETASIMAX,SQS,
     & PSIDD1,PSIDD2
      COMMON/EDDEHARD/ MGGCUT,ETJCUT,MXMAX,
     & ETAJMAX,PLUM,PSURV,PSUD,ETASIMAX,SQS,
     & PSIDD1,PSIDD2
c--- restrictions on the phase space of g-jet ---
c--- DER3J - max. angle between g-jet and parallel jet ----
c--- XMAX3J - max ratio 2*Eg/MJJ --------------------------
c--- parameters for 3g functions -----------------------------
      DOUBLE PRECISION DER3J,XMAX3J,PAR3G
      COMMON/EDDE3JP/ DER3J,XMAX3J,PAR3G(5)     
c----- parameters to calculate total cross-sections -------      
c----- RS1 parameters -------------------------------------
      INTEGER NRS0 
      DOUBLE PRECISION RSXI0,RSGAM0,RSMH0,RSMR0 
      COMMON/EDDERS1/ RSXI0,RSGAM0,RSMH0,RSMR0,NRS0
c----- additional global parameters -----
      INTEGER KCP,IPROC
      DOUBLE PRECISION AM0,AMP,S,MQ
      COMMON/EDDEOTHER/ KCP,IPROC,AM0,AMP,S,MQ
c-...global EDDE parameters from FFR file
c      INTEGER MXGLPAR
c      INTEGER IPARS
c      REAL RPARS
c      PARAMETER   (MXGLPAR=200)
c      COMMON /KI_TXPAR/ IPARS(MXGLPAR), RPARS(MXGLPAR)

      LOGICAL FIRST
      DATA FIRST /.TRUE./
      SAVE FIRST

      IF(FIRST) THEN
       KOD   =IPARS(30)
       EMIN  =RPARS(30)
       EMAX  =RPARS(31)
       ETAMIN=RPARS(32)
       ETAMAX=RPARS(33)
      ENDIF     
      
      E=PYR(0)*(EMAX-EMIN)
      ETA=ETAMIN+PYR(0)*(ETAMAX-ETAMIN)
      TET=2*DATAN(EXP(-ETA))
      FI  = 2.*PI*PYR(0)
      
      CALL PY1ENT(3,KOD,E,TET,FI)

      RETURN
      END
